<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request\DataSet;
use HCM\Util\HCMParameters;

class ProcessoAdministrativoController extends Crud {

    protected $dataSourceNameGet = 'getProcAdministrativo';
    protected $dataSourceNameSet = 'fpaProcAdministrativo';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC', 'P_NREVENTOM');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getProcAdministrativo(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setProcessoAdministrativo (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if($row['__is_new'] === true){
                    $row['NRPROCINCIDEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_PROCINCIDEVENTO', 1, 12, true);
                }
                
                $row["NRINCIDCP"] = $row["NRINCIDCP"]??null;
                $row["NRINCIDIRRF"] = $row["NRINCIDIRRF"]??null;
                $row["NRINCIDFGTS"] = $row["NRINCIDFGTS"]??null;
            }
            $this->dataSourceName = $this->dataSourceNameSet;
            parent::save($request, $response);
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function deleteProcessoAdministrativo(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceNameSet;
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error(empty($error) ? $e->getMessage() : $error, empty($error) ? $e->getCode() : 5));
        }  
    }
    
    public function populaParamsGetRegrasEvento($conditions){
        $conditionsReturn = array();
        foreach($conditions as $condition){
            $conditionsReturn[$condition['columnName']] = $condition['value'];
        }
        return $conditionsReturn;
    }
    
    // public function getRegrasEvento($params){
    //     return $this->regraService->getRegrasEvento($params);
    // }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 