<?php

namespace HCM\FPA\ObrigacaoLegal\SEFIP;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\Geral\Entity\GpevinculoempregEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\FPA\Registro\sefip\Registro00SEFIP;
use HCM\FPA\Registro\sefip\Registro90SEFIP;
use HCM\FPA\Registro\RegistroSEFIP;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaGrpitemEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpagfipmagnetEntity;

class PercorreSEFIP{

    const CLT = 1;
    const SOCIO = 3;
    const ESTAGIARIO = 2;

    private $_parametros;
    private $_formulario;
    private $_retrocesso;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;
    private $_nrTipoModalidadeCalculo;
    private $_tipoMovimento;
    private $_listaTipoMovimento;
    private $_listaTipoRegistro;
    private $_dsInicio;
    private $_dsFim;
    private $_dsOrdem;
    private $_nrTpHierarquiaOrg;
    private $_tpInscHierarq;
    private $_nrTpHierarqTomador;
    private $_tpInscHierarqTomador;
    private $_idProcessaVinculo;
    private $_idGeraMeioMagnetico;
    private $_filePath;
    private $_nrEstruturaResp;
    private $_tpInscResp;
    private $_nrPessoaResp;
    private $_idRecolhimentoFGTS;
    private $_dtFGTS;
    private $_idRecolhimentoGPS;
    private $_dtGPS;
    private $_nrGrpIncide;
    private $_nrOcorrencia;
    private $_calculoFolha;
    private $_listaNrCalculoFolha;
    private $_gfipMagnetico;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_vinculom;
    private $_itemGrupo;
    private $_pessoa;
    private $_estruturacao;
    private $_sefip;
    private $_registro00;
    private $_registro90;
    private $_nrRecolhimento;
    private $_idModalidade;
    private $_competencia13;
    private $_msg_idstatus;
    private $_msg_idabort;
    private $_msg_mensagens;
    private $_cdOperador;

    public function __construct($entityManager, $hcmParameters, $dataSet){

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdOperador();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_nrTipoModalidadeCalculo = $hcmParameters->getTipoModalidadeCalculo();
        $this->_tipoMovimento = FpatipomovimentEntity::TP_MOV_MENSAL;
        $this->_listaTipoMovimento[] = FpatipomovimentEntity::TP_MOV_MENSAL;
        $this->_listaTipoMovimento[] = FpatipomovimentEntity::TP_MOV_13SAL;
        $this->_itemGrupo = new FpaGrpitemEntity($entityManager, $hcmParameters);
        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_vinculoEmpreg = new GpevinculoempregEntity($entityManager, $hcmParameters);
        $this->_tipoHierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_hierarquiaOrg = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_calculoFolha = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        $this->_gfipMagnetico = new FpagfipmagnetEntity($entityManager, $hcmParameters);
        $this->_registro00 = new Registro00SEFIP();
        $this->_registro90 = new Registro90SEFIP();
        $this->_pessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);

        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch (strtoupper($tp_processamento)){
            case FPA::PROC_COMPETENCIA_MENSAL :
                $this->_retrocesso = false;
                $this->_competencia13 = false;
                break;
            case FPA::PROC_COMPETENCIA_13 :
                $this->_retrocesso = false;
                $this->_competencia13 = true;
                break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true;
                $this->_competencia13 = false;
                break;
            default:
                throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');
        }

        $this->_inicializaVariaveis($dataSet);

        $this->_sefip  = new SEFIP(
            $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_gfipMagnetico, $dataSet,
            $this->_competencia13, $this->_itemGrupo, $this->_listaNrCalculoFolha, $this->entityManager, $this->hcmParameters
        );
    }


    public function _inicializaVariaveis($dataSet) {

        $this->_buscaDadosFormulario($dataSet);
        $this->_itemGrupo->carregaItensGrupoPorGrupoIncidencia($this->_nrOrg, $this->_nrOrgPadrao, $this->_nrGrpIncide);
        
        foreach ($this->_listaTipoMovimento as $tipoMovimento) {
            if (!$this->_nrOcorrencia) {
                $listaNrCalculo = $this->_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias(
                    $this->_nrOrg, $tipoMovimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo
                );
                    
                foreach ($listaNrCalculo as $nrCalculo) {
                    $this->_listaNrCalculoFolha[] = $nrCalculo;
                }
            } else {
                $arrayOcorrencias = explode('.', $this->_nrOcorrencia);
                
                foreach ($arrayOcorrencias as $nrocorrencia) {
                    $calculos = $this->_calculoFolha->getCalculoFolha(
                        $this->_nrOrg, $tipoMovimento, $this->_nrTipoModalidadeCalculo, abs($nrocorrencia), $this->_mesCompetencia
                    );
                    if ($calculos) {
                        foreach ($calculos as $calculo) {
                            $this->_listaNrCalculoFolha[] = $calculo->getNrcalculofolha();
                        }
                    }
                }
                if (empty($this->_listaNrCalculoFolha)) {
                   throw new \Exception("Não foram encontrados cálculos para as ocorrências informadas.");
                }
            }
        }
    }

    public function percorre(){
        if ($this->_retrocesso === false) {
            try{
                $registros = $this->_gfipMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg, $this->_cdOperador);
                if (count($registros) == 0) {
                    $resultado = $this->_geraSEFIP();
                    return $resultado;
                } else {
                    throw new \Exception('Já existe magnético do Sefip para o mês de competência, favor retroceder.');
                }
            }catch (Exception $e) {
                throw new \Exception($e->getMessage(), $e->getCode());
            }
        } else {
            try {
                $this->_sefip->retroceder();
                return array("RETROCESSO" => true);
            } catch (Exception $e) {
                throw new \Exception($e->getMessage(), $e->getCode());
            }
        }
    }

    private function _geraSEFIP(){

        $this->_msg_idabort = false;

        if ($this->_idProcessaVinculo == 'S') {

            $adicionou_itens = $this->_gfipMagnetico->inicializaListaRegistros(
                $this->_mesCompetencia, RegistroSEFIP::LISTA_TIPO_REG_EMP, $this->_nrOrg, $this->_cdOperador
            );

            $vinculos_ativos = $this->_vinculom->retornaVinculosAtivosMesCompetencia($this->_nrOrg, $this->_mesCompetencia,
                $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, true, $this->_nrEstruturaTrab);
                
            $vinculos_compl = $this->_vinculom->retornaVinculosRescisaoComplementarCompetencia($this->_nrOrg, $this->_mesCompetencia,
                $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, null, null, $this->_nrEstruturaTrab);
            
            $vinculos = array_merge($vinculos_ativos, $vinculos_compl);

            $this->_msg_idstatus = MsgCalculo::ID_PENDENTE;
            $nrcalculomsg = $this->_listaNrCalculoFolha[0];

            if ($this->_competencia13) {
                $listaVinculosSemLiq = null;
			} else {
                $listaVinculosSemLiq = $this->_vinculom->verificaLiquidoVinculos($this->_nrOrg, $this->_mesCompetencia, FpatipomovimentEntity::TP_MOV_MENSAL, $vinculos, null, null, null, true, false);
            }
            
            if (is_array($listaVinculosSemLiq)) {
                foreach ($listaVinculosSemLiq as $vinculoSemLiq) {

                    if ($vinculoSemLiq->getDtrescisaovinc() != null){
                        $dtRescisaoVinc = DateUtil::getDataDeString($vinculoSemLiq->getDtrescisaovinc());
                    }

                    $processamento = MsgCalculo::getInstance();
                    $processamento->setDados($nrcalculomsg, $vinculoSemLiq->getNrvinculom(), $this->_nrOrg, 'SEFIP');

                    if (($vinculoSemLiq->getDtrescisaovinc() == null) or ($dtRescisaoVinc > DateUtil::getUltimoDiaDoMes($this->_mesCompetencia))) {
                        $mensagem = 'Vínculo ' . $vinculoSemLiq->getNrvinculom() . '-' . $vinculoSemLiq->getNmvinculom() . ' sem líquido.';
                    } else {
                        $mensagem = 'Vínculo ' . $vinculoSemLiq->getNrvinculom() . '-' . $vinculoSemLiq->getNmvinculom() . ' sem líquido de rescisão.';
                    }
                    
                    MsgCalculo::getInstance()->adicionaMensagem($mensagem, MsgCalculo::ERRO, true);
                    
                    $this->finalizaMensagens();
                }
            }
            else {
                foreach ($vinculos as $vinculo) {

                    $processamento = MsgCalculo::getInstance();
                    $processamento->setDados($nrcalculomsg, $vinculo->getNrvinculom(), $this->_nrOrg, 'SEFIP');

                    if ($vinculo->getNrtipovinculom() == self::ESTAGIARIO) {
                        continue;
                    }

                    if ($vinculo->getNrvinculoempreg() !== false) {

                        $array_categ = array('01', '04', '07', '12', '19', '20', '21', '26');
                        
                        $vinculoEmpreg = $this->_vinculoEmpreg->retornaVinculoEmpregaticio($vinculo->getNrvinculoempreg());
                        
                        if (($this->_competencia13) && (!in_array($vinculoEmpreg->getCodcategtrab(), $array_categ))) {
                            continue;
                        }

                        $this->_sefip->geraSEFIP($vinculo, $this->_nrTpHierarquiaOrg, $this->_nrTpHierarqTomador, $this->_tpInscHierarq, $this->_tpInscHierarqTomador);
                    } else {
                        MsgCalculo::getInstance()->adicionaMensagem('Vinculo empregaticio nao encontrado para o vinculo ' . $vinculo->getNrvinculom(), MsgCalculo::ERRO);
                    }

                    if (!$this->finalizaMensagens()) {
                        continue;
                    }

                }

                $this->_sefip->modificaValoresConsolidados();
                $qtdRegistros = $this->_gfipMagnetico->countItems();
                if ($qtdRegistros > 0) {
                    $this->_gfipMagnetico->persist();
                }
            }
        }
        if (($this->_msg_idabort == false) and ($this->_msg_idstatus !== MsgCalculo::ID_COM_ERRO)) {
            if ($this->_idGeraMeioMagnetico == 'S') {
                $arquivo = $this->_geraMagnetico();
            }
        }
        
        return array(
            'ARQUIVO' => $arquivo,
            'CALCULO' => $nrcalculomsg,
            'ERRO' => (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)),
            'ABORT' => $this->_msg_idabort
        );
    }

    private function _geraMagnetico($tpInscricao = null, $nrInscricao = null){

        $nomeArquivo = "SEFIP.RE";

        $this->_preparaRegistro00();

        $this->_registro00->geraRegistro00();
        
        FPA::escreveArquivo($nomeArquivo, $this->_registro00->getLinhaRegistro().RegistroSEFIP::QUEBRA_LINHA_CR_LF);    
        
        $resultado = $this->_gfipMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg, $this->_cdOperador);
        
        foreach ($resultado as $item) {
            FPA::escreveArquivo($nomeArquivo, $item->getDslinharegistro().RegistroSEFIP::QUEBRA_LINHA_CR_LF);
        }

        $this->_registro90->geraRegistro90();
    
        FPA::escreveArquivo($nomeArquivo, $this->_registro90->getLinhaRegistro());
        
        return $nomeArquivo;
    }

    private function _preparaRegistro00(){

        $estrutura = $this->_estruturacao->retornaEstrutura($this->_nrEstruturaResp);
        if (is_object($estrutura) === true){
            $estrutura->endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $pessoa = $this->_pessoa->retornaPessoaAtual($this->_nrPessoaResp);

            if ($this->_tpInscResp === RegistroSEFIP::CNPJ) {
                $this->_registro00->setTpInscricao(RegistroSEFIP::CNPJ);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdcnpjestrut());
            } elseif ($this->_tpInscResp === RegistroSEFIP::CEI) {
                $this->_registro00->setTpInscricao(RegistroSEFIP::CEI);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdceiestrut());
            }
            $this->_registro00->setPessoaResp($pessoa);
            $this->_registro00->setEstruturaResp($estrutura);
            $this->_registro00->setCompetencia($this->_mesCompetencia);
            $this->_registro00->setCodigoRecolhimento($this->_nrRecolhimento);
            $this->_registro00->setModalidade($this->_idModalidade);
            $this->_registro00->setIdRecolhimentoFGTS($this->_idRecolhimentoFGTS);
            $this->_registro00->setDtFGTS($this->_dtFGTS);
            $this->_registro00->setIdRecolhimentoGPS($this->_idRecolhimentoGPS);
            $this->_registro00->setDtGPS($this->_dtGPS);
            $this->_registro00->setCompetencia13($this->_competencia13);
        }
    }

    private function _buscaDadosFormulario($dataSet){
        
        if ($dataSet) {
            $this->_nrRecolhimento = $dataSet['NRRECOLHIMENTO'];
            $this->_idModalidade = $dataSet['IDMODALIDADE'];
            $this->_dsInicio = $dataSet['DSINICIO'];
            $this->_dsFim = $dataSet['DSFIM'];
            $this->_dsOrdem = $dataSet['DSORDEM'];
            $this->_nrTpHierarquiaOrg = $dataSet['NRHIERARQORG'];
            $this->_tpInscHierarq = $dataSet['IDTPINSCHIERARQ'];
            $this->_nrTpHierarqTomador = $dataSet['NRHIERARTOMADOR'];
            $this->_tpInscHierarqTomador = $dataSet['IDTPINSCHIERTOM'];
            $this->_idProcessaVinculo = $dataSet['IDPROCVINCULO'];
            $this->_idGeraMeioMagnetico = $dataSet['IDGERAMEIOMAG'];
            $this->_filePath = $dataSet['DSCAMINHOARQUIVO'];
            $this->_nrEstruturaResp = $dataSet['NRESTRUTRESPONS'];
            $this->_tpInscResp = $dataSet['IDTPINSCRESPONS'];
            $this->_nrPessoaResp = $dataSet['NRPESSOARESPONS'];
            $this->_idRecolhimentoFGTS = $dataSet['IDRECOLHFGTS'];
            $this->_dtFGTS = $dataSet['DTRECOLHFGTS'];
            $this->_idRecolhimentoGPS = $dataSet['IDRECOLHGPS'];
            $this->_dtGPS = $dataSet['DTRECOLHGPS'];
            $this->_nrGrpIncide = $dataSet['NRGRPINCIDE'];
            $this->_nrOcorrencia = $dataSet['NROCORRENCIA'];
        } else {
            throw new \Exception("Formulário não encontrado, contate o administrador.");
        }
    }

    private function finalizaMensagens() {
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            MsgCalculo::getInstance()->salvaMensagens();
            MsgCalculo::getInstance()->clear();
            return false;
        }
     
        MsgCalculo::getInstance()->salvaMensagens();
        MsgCalculo::getInstance()->clear();
        return true;
    }
}