<?php

namespace HCM\Geral\Logic\Esocial;

use HCM\Geral\Logic\ESocialLogic;

class ESocialTransmissor {
    
    const TPAMB_PROD = 1;
    const TPAMB_PROD_REST_REAL = 2;
    
    const URL_ENVIO_PRODUCAO = 'https://webservices.envio.esocial.gov.br';
    const URL_CONSULTA_PRODUCAO = 'https://webservices.consulta.esocial.gov.br';
    const URL_DOWNLOAD_PRODUCAO = 'https://webservices.download.esocial.gov.br';
    
    const URL_PRODREST = 'https://webservices.producaorestrita.esocial.gov.br';
    
    const LOCATION_ENVIO = '/servicos/empregador/enviarloteeventos/WsEnviarLoteEventos.svc';
    const LOCATION_CONSULTA = '/servicos/empregador/consultarloteeventos/WsConsultarLoteEventos.svc';
    const LOCATION_CONSULTA_ID = '/servicos/empregador/dwlcirurgico/WsConsultarIdentificadoresEventos.svc';
    const LOCATION_DOWNLOAD_EVENTOS = '/servicos/empregador/dwlcirurgico/WsSolicitarDownloadEventos.svc';
    
    const FUNCTION_ENVIO = 'EnviarLoteEventos';
    const FUNCTION_CONSULTA = 'ConsultarLoteEventos';
    const FUNCTION_CONSULTA_ID_EMPR = 'ConsultarIdentificadoresEventosEmpregador';
    const FUNCTION_CONSULTA_ID_TABE = 'ConsultarIdentificadoresEventosTabela';
    const FUNCTION_CONSULTA_ID_TRAB = 'ConsultarIdentificadoresEventosTrabalhador';
    const FUNCTION_DOWNLOAD_EV_ID = 'SolicitarDownloadEventosPorId';
    const FUNCTION_DOWNLOAD_EV_REC = 'SolicitarDownloadEventosPorNrRecibo';
    
    private $wsLocation;
    private $wsUri;
    private $wsWsdl;
    private $wsFuncion;

    public function __construct($tpAmbiente, $idOperacao, $certKeyFile)
    {
        if ($tpAmbiente == self::TPAMB_PROD) {
            switch ($idOperacao) {
                case ESocialLogic::IDTRANSENVIO:
                    $this->wsLocation = self::URL_ENVIO_PRODUCAO . self::LOCATION_ENVIO;
                    $this->wsUri = self::URL_ENVIO_PRODUCAO;
                    $this->wsWsdl = self::URL_ENVIO_PRODUCAO . self::LOCATION_ENVIO . '?wsdl';
                    $this->wsFuncion = self::FUNCTION_ENVIO;
                    break;
                case ESocialLogic::IDTRANSCONSU:
                    $this->wsLocation = self::URL_CONSULTA_PRODUCAO . self::LOCATION_CONSULTA;
                    $this->wsUri = self::URL_CONSULTA_PRODUCAO;
                    $this->wsWsdl = self::URL_CONSULTA_PRODUCAO . self::LOCATION_CONSULTA . '?wsdl';
                    $this->wsFuncion = self::FUNCTION_CONSULTA;
                    break;
                case ESocialLogic::IDCONSUIDEMPR:
                case ESocialLogic::IDCONSUIDTABE:
                case ESocialLogic::IDCONSUIDTRAB:
                    $this->wsLocation = self::URL_DOWNLOAD_PRODUCAO . self::LOCATION_CONSULTA_ID;
                    $this->wsUri = self::URL_DOWNLOAD_PRODUCAO;
                    $this->wsWsdl = self::URL_DOWNLOAD_PRODUCAO . self::LOCATION_CONSULTA_ID . '?wsdl';
                    if ($idOperacao == ESocialLogic::IDCONSUIDEMPR) {
                        $this->wsFuncion = self::FUNCTION_CONSULTA_ID_EMPR;
                    }
                    if ($idOperacao == ESocialLogic::IDCONSUIDTABE) {
                        $this->wsFuncion = self::FUNCTION_CONSULTA_ID_TABE;
                    }
                    if ($idOperacao == ESocialLogic::IDCONSUIDTRAB) {
                        $this->wsFuncion = self::FUNCTION_CONSULTA_ID_TRAB;
                    }
                    break;
                case ESocialLogic::IDDOWNLOADID:
                case ESocialLogic::IDDOWNLOADREC:
                    $this->wsLocation = self::URL_DOWNLOAD_PRODUCAO . self::LOCATION_DOWNLOAD_EVENTOS;
                    $this->wsUri = self::URL_DOWNLOAD_PRODUCAO;
                    $this->wsWsdl = self::URL_DOWNLOAD_PRODUCAO . self::LOCATION_DOWNLOAD_EVENTOS . '?wsdl';
                    if ($idOperacao == ESocialLogic::IDDOWNLOADID) {
                        $this->wsFuncion = self::FUNCTION_DOWNLOAD_EV_ID;
                    }
                    if ($idOperacao == ESocialLogic::IDDOWNLOADREC) {
                        $this->wsFuncion = self::FUNCTION_DOWNLOAD_EV_REC;
                    }
                    break;
                default:
                    $this->wsLocation = '';
                    $this->wsUri = '';
                    $this->wsWsdl = '';
                    $this->wsFuncion = '';
            }
        } else {
            switch ($idOperacao) {
                case ESocialLogic::IDTRANSENVIO:
                    $this->wsLocation = self::URL_PRODREST . self::LOCATION_ENVIO;
                    $this->wsUri = self::URL_PRODREST;
                    $this->wsWsdl = self::URL_PRODREST . self::LOCATION_ENVIO . '?wsdl';
                    $this->wsFuncion = self::FUNCTION_ENVIO;
                    break;
                case ESocialLogic::IDTRANSCONSU:
                    $this->wsLocation = self::URL_PRODREST . self::LOCATION_CONSULTA;
                    $this->wsUri = self::URL_PRODREST;
                    $this->wsWsdl = self::URL_PRODREST . self::LOCATION_CONSULTA . '?wsdl';
                    $this->wsFuncion = self::FUNCTION_CONSULTA;
                    break;
                case ESocialLogic::IDCONSUIDEMPR:
                case ESocialLogic::IDCONSUIDTABE:
                case ESocialLogic::IDCONSUIDTRAB:
                    $this->wsLocation = self::URL_PRODREST . self::LOCATION_CONSULTA_ID;
                    $this->wsUri = self::URL_PRODREST;
                    $this->wsWsdl = self::URL_PRODREST . self::LOCATION_CONSULTA_ID . '?wsdl';
                    if ($idOperacao == ESocialLogic::IDCONSUIDEMPR) {
                        $this->wsFuncion = self::FUNCTION_CONSULTA_ID_EMPR;
                    }
                    if ($idOperacao == ESocialLogic::IDCONSUIDTABE) {
                        $this->wsFuncion = self::FUNCTION_CONSULTA_ID_TABE;
                    }
                    if ($idOperacao == ESocialLogic::IDCONSUIDTRAB) {
                        $this->wsFuncion = self::FUNCTION_CONSULTA_ID_TRAB;
                    }
                    break;
                case ESocialLogic::IDDOWNLOADID:
                case ESocialLogic::IDDOWNLOADREC:
                    $this->wsLocation = self::URL_PRODREST . self::LOCATION_DOWNLOAD_EVENTOS;
                    $this->wsUri = self::URL_PRODREST;
                    $this->wsWsdl = self::URL_PRODREST . self::LOCATION_DOWNLOAD_EVENTOS . '?wsdl';
                    if ($idOperacao == ESocialLogic::IDDOWNLOADID) {
                        $this->wsFuncion = self::FUNCTION_DOWNLOAD_EV_ID;
                    }
                    if ($idOperacao == ESocialLogic::IDDOWNLOADREC) {
                        $this->wsFuncion = self::FUNCTION_DOWNLOAD_EV_REC;
                    }
                    break;
                default:
                    $this->wsLocation = '';
                    $this->wsUri = '';
                    $this->wsWsdl = '';
                    $this->wsFuncion = '';
            }
        }
        
        $this->_certKeyFile = $certKeyFile;
    }
    
    /**
     * @return \SoapClient
     * @throws \Exception
     */
    private function buildSoapCall($content) {
        try {
            $soapVar = new \SoapVar($content, XSD_ANYXML);
            
            $options = array(
                'local_cert' => $this->_certKeyFile,
                'trace' => 1,
                'location' => $this->wsLocation,
                'uri' => $this->wsUri,
                //'crypto_method' =>  STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT,
                'soap_version'   => SOAP_1_1,
                'stream_context' => stream_context_create(array(
                    'http' => array(
                        'user_agent' => 'PHP/SOAP',
                        'header' => "Accept: application/xml"
                    ),
                    'ssl' => array(
                        'verify_peer' => false,
                        'verify_peer_name' => false,
                        'allow_self_signed' => true,
                    )
                )),
                'exceptions'     => true,
                //'proxy_host'     => "192.168.122.3",
                //'proxy_port'     => 8080,
                //'proxy_login'    => "login",
                //'proxy_password' => "password",
                'connection_timeout' => 120,
            );

            $client = new \SoapClient($this->wsWsdl, $options);
            
            ini_set('default_socket_timeout', 5000);
            ini_set('soap.wsdl_cache_enabled',0);
            ini_set('soap.wsdl_cache_ttl',0);
            $result = $client->__soapCall($this->wsFuncion, array($soapVar));

            if (get_class($result) == 'SoapFault') {
                /** @var \SoapFault $result */
                throw new \Exception($result->getMessage());
            }

            unset($client);
            return $result;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $xml
     * @return string
     * @throws \Exception
     */
    public function transmitirXml($xml) {
        try {
            $result = $this->buildSoapCall($xml);

            $functionResult = $this->wsFuncion . 'Result';
            if ($result->$functionResult) {
                $xmlResult = $result->$functionResult->any;
                return $xmlResult;
            }  else {
                throw new \Exception($result->__toString());
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

}